@@ -5,14 +5,17 @@
 Test for L{twisted.web.proxy}.
 """
 
+from twisted.test.proto_helpers import MemoryReactor, StringTransportWithDisconnection
 from twisted.trial.unittest import TestCase
-from twisted.test.proto_helpers import StringTransportWithDisconnection
-from twisted.test.proto_helpers import MemoryReactor
-
+from twisted.web.proxy import (
+    ProxyClient,
+    ProxyClientFactory,
+    ProxyRequest,
+    ReverseProxyRequest,
+    ReverseProxyResource,
+)
 from twisted.web.resource import Resource
 from twisted.web.server import Site
-from twisted.web.proxy import ReverseProxyResource, ProxyClientFactory
-from twisted.web.proxy import ProxyClient, ProxyRequest, ReverseProxyRequest
 from twisted.web.test.test_web import DummyRequest
 
 
@@ -28,8 +31,8 @@ class ReverseProxyResourceTests(TestCase):
         """
         root = Resource()
         reactor = MemoryReactor()
-        resource = ReverseProxyResource(u"127.0.0.1", 1234, b"/path", reactor)
-        root.putChild(b'index', resource)
+        resource = ReverseProxyResource("127.0.0.1", 1234, b"/path", reactor)
+        root.putChild(b"index", resource)
         site = Site(root)
 
         transport = StringTransportWithDisconnection()
@@ -38,13 +41,11 @@ class ReverseProxyResourceTests(TestCase):
         # Clear the timeout if the tests failed
         self.addCleanup(channel.connectionLost, None)
 
-        channel.dataReceived(b"GET " +
-                             uri +
-                             b" HTTP/1.1\r\nAccept: text/html\r\n\r\n")
+        channel.dataReceived(b"GET " + uri + b" HTTP/1.1\r\nAccept: text/html\r\n\r\n")
 
         [(host, port, factory, _timeout, _bind_addr)] = reactor.tcpClients
         # Check that one connection has been created, to the good host/port
-        self.assertEqual(host, u"127.0.0.1")
+        self.assertEqual(host, "127.0.0.1")
         self.assertEqual(port, 1234)
 
         # Check the factory passed to the connect, and its given path
@@ -52,7 +53,6 @@ class ReverseProxyResourceTests(TestCase):
         self.assertEqual(factory.rest, expectedURI)
         self.assertEqual(factory.headers[b"host"], b"127.0.0.1:1234")
 
-
     def test_render(self):
         """
         Test that L{ReverseProxyResource.render} initiates a connection to the
@@ -60,7 +60,6 @@ class ReverseProxyResourceTests(TestCase):
         """
         return self._testRender(b"/index", b"/path")
 
-
     def test_render_subpage(self):
         """
         Test that L{ReverseProxyResource.render} will instantiate a child
@@ -69,7 +68,6 @@ class ReverseProxyResourceTests(TestCase):
         """
         return self._testRender(b"/index/page1", b"/path/page1")
 
-
     def test_renderWithQuery(self):
         """
         Test that L{ReverseProxyResource.render} passes query parameters to the
@@ -77,7 +75,6 @@ class ReverseProxyResourceTests(TestCase):
         """
         return self._testRender(b"/index?foo=bar", b"/path?foo=bar")
 
-
     def test_getChild(self):
         """
         The L{ReverseProxyResource.getChild} method should return a resource
@@ -86,28 +83,26 @@ class ReverseProxyResourceTests(TestCase):
         value passed.
         """
         reactor = MemoryReactor()
-        resource = ReverseProxyResource(u"127.0.0.1", 1234, b"/path", reactor)
-        child = resource.getChild(b'foo', None)
+        resource = ReverseProxyResource("127.0.0.1", 1234, b"/path", reactor)
+        child = resource.getChild(b"foo", None)
         # The child should keep the same class
         self.assertIsInstance(child, ReverseProxyResource)
         self.assertEqual(child.path, b"/path/foo")
         self.assertEqual(child.port, 1234)
-        self.assertEqual(child.host, u"127.0.0.1")
+        self.assertEqual(child.host, "127.0.0.1")
         self.assertIdentical(child.reactor, resource.reactor)
 
-
     def test_getChildWithSpecial(self):
         """
         The L{ReverseProxyResource} return by C{getChild} has a path which has
         already been quoted.
         """
-        resource = ReverseProxyResource(u"127.0.0.1", 1234, b"/path")
-        child = resource.getChild(b' /%', None)
+        resource = ReverseProxyResource("127.0.0.1", 1234, b"/path")
+        child = resource.getChild(b" /%", None)
         self.assertEqual(child.path, b"/path/%20%2F%25")
 
 
-
-class DummyChannel(object):
+class DummyChannel:
     """
     A dummy HTTP channel, that does nothing but holds a transport and saves
     connection lost.
@@ -123,21 +118,18 @@ class DummyChannel(object):
         self.transport = transport
         self.lostReason = None
 
-
     def connectionLost(self, reason):
         """
         Keep track of the connection lost reason.
         """
         self.lostReason = reason
 
-
     def getPeer(self):
         """
         Get peer information from the transport.
         """
         return self.transport.getPeer()
 
-
     def getHost(self):
         """
         Get host information from the transport.
@@ -145,7 +137,6 @@ class DummyChannel(object):
         return self.transport.getHost()
 
 
-
 class ProxyClientTests(TestCase):
     """
     Tests for L{ProxyClient}.
@@ -160,12 +151,10 @@ class ProxyClientTests(TestCase):
             of headers, C{requestLine} is the first line (e.g. "POST /foo ...")
             and C{body} is whatever is left.
         """
-        headers, body = content.split(b'\r\n\r\n')
-        headers = headers.split(b'\r\n')
+        headers, body = content.split(b"\r\n\r\n")
+        headers = headers.split(b"\r\n")
         requestLine = headers.pop(0)
-        return (
-            requestLine, dict(header.split(b': ') for header in headers), body)
-
+        return (requestLine, dict(header.split(b": ") for header in headers), body)
 
     def makeRequest(self, path):
         """
@@ -176,9 +165,7 @@ class ProxyClientTests(TestCase):
         """
         return DummyRequest(path)
 
-
-    def makeProxyClient(self, request, method=b"GET", headers=None,
-                        requestBody=b""):
+    def makeProxyClient(self, request, method=b"GET", headers=None, requestBody=b""):
         """
         Make a L{ProxyClient} object used for testing.
 
@@ -192,10 +179,8 @@ class ProxyClientTests(TestCase):
         """
         if headers is None:
             headers = {b"accept": b"text/html"}
-        path = b'/' + request.postpath
-        return ProxyClient(
-            method, path, b'HTTP/1.0', headers, requestBody, request)
-
+        path = b"/" + request.postpath
+        return ProxyClient(method, path, b"HTTP/1.0", headers, requestBody, request)
 
     def connectProxy(self, proxyClient):
         """
@@ -209,7 +194,6 @@ class ProxyClientTests(TestCase):
         proxyClient.makeConnection(clientTransport)
         return clientTransport
 
-
     def assertForwardsHeaders(self, proxyClient, requestLine, headers):
         """
         Assert that C{proxyClient} sends C{headers} when it connects.
@@ -222,21 +206,18 @@ class ProxyClientTests(TestCase):
         """
         self.connectProxy(proxyClient)
         requestContent = proxyClient.transport.value()
-        receivedLine, receivedHeaders, body = self._parseOutHeaders(
-            requestContent)
+        receivedLine, receivedHeaders, body = self._parseOutHeaders(requestContent)
         self.assertEqual(receivedLine, requestLine)
         self.assertEqual(receivedHeaders, headers)
         return body
 
-
     def makeResponseBytes(self, code, message, headers, body):
-        lines = [b"HTTP/1.0 " + str(code).encode('ascii') + b' ' + message]
+        lines = [b"HTTP/1.0 " + str(code).encode("ascii") + b" " + message]
         for header, values in headers:
             for value in values:
-                lines.append(header + b': ' + value)
-        lines.extend([b'', body])
-        return b'\r\n'.join(lines)
-
+                lines.append(header + b": " + value)
+        lines.extend([b"", body])
+        return b"\r\n".join(lines)
 
     def assertForwardsResponse(self, request, code, message, headers, body):
         """
@@ -255,28 +236,37 @@ class ProxyClientTests(TestCase):
         expectedHeaders = headers[:]
         expectedHeaders.sort()
         self.assertEqual(receivedHeaders, expectedHeaders)
-        self.assertEqual(b''.join(request.written), body)
+        self.assertEqual(b"".join(request.written), body)
 
-
-    def _testDataForward(self, code, message, headers, body, method=b"GET",
-                         requestBody=b"", loseConnection=True):
+    def _testDataForward(
+        self,
+        code,
+        message,
+        headers,
+        body,
+        method=b"GET",
+        requestBody=b"",
+        loseConnection=True,
+    ):
         """
         Build a fake proxy connection, and send C{data} over it, checking that
         it's forwarded to the originating request.
         """
-        request = self.makeRequest(b'foo')
+        request = self.makeRequest(b"foo")
         client = self.makeProxyClient(
-            request, method, {b'accept': b'text/html'}, requestBody)
+            request, method, {b"accept": b"text/html"}, requestBody
+        )
 
         receivedBody = self.assertForwardsHeaders(
-            client, method + b' /foo HTTP/1.0',
-            {b'connection': b'close', b'accept': b'text/html'})
+            client,
+            method + b" /foo HTTP/1.0",
+            {b"connection": b"close", b"accept": b"text/html"},
+        )
 
         self.assertEqual(receivedBody, requestBody)
 
         # Fake an answer
-        client.dataReceived(
-            self.makeResponseBytes(code, message, headers, body))
+        client.dataReceived(self.makeResponseBytes(code, message, headers, body))
 
         # Check that the response data has been forwarded back to the original
         # requester.
@@ -292,7 +282,6 @@ class ProxyClientTests(TestCase):
         self.assertFalse(client.transport.connected)
         self.assertEqual(request.finished, 1)
 
-
     def test_forward(self):
         """
         When connected to the server, L{ProxyClient} should send the saved
@@ -300,8 +289,8 @@ class ProxyClientTests(TestCase):
         to the parent request.
         """
         return self._testDataForward(
-            200, b"OK", [(b"Foo", [b"bar", b"baz"])], b"Some data\r\n")
-
+            200, b"OK", [(b"Foo", [b"bar", b"baz"])], b"Some data\r\n"
+        )
 
     def test_postData(self):
         """
@@ -309,17 +298,15 @@ class ProxyClientTests(TestCase):
         forward the body of the request.
         """
         return self._testDataForward(
-            200, b"OK", [(b"Foo", [b"bar"])], b"Some data\r\n", b"POST", b"Some content")
-
+            200, b"OK", [(b"Foo", [b"bar"])], b"Some data\r\n", b"POST", b"Some content"
+        )
 
     def test_statusWithMessage(self):
         """
         If the response contains a status with a message, it should be
         forwarded to the parent request with all the information.
         """
-        return self._testDataForward(
-            404, b"Not Found", [], b"")
-
+        return self._testDataForward(404, b"Not Found", [], b"")
 
     def test_contentLength(self):
         """
@@ -328,11 +315,8 @@ class ProxyClientTests(TestCase):
         """
         data = b"foo bar baz"
         return self._testDataForward(
-            200,
-            b"OK",
-            [(b"Content-Length", [str(len(data)).encode('ascii')])],
-            data)
-
+            200, b"OK", [(b"Content-Length", [str(len(data)).encode("ascii")])], data
+        )
 
     def test_losesConnection(self):
         """
@@ -343,10 +327,10 @@ class ProxyClientTests(TestCase):
         return self._testDataForward(
             200,
             b"OK",
-            [(b"Content-Length", [str(len(data)).encode('ascii')])],
+            [(b"Content-Length", [str(len(data)).encode("ascii")])],
             data,
-            loseConnection=False)
-
+            loseConnection=False,
+        )
 
     def test_headersCleanups(self):
         """
@@ -354,11 +338,17 @@ class ProxyClientTests(TestCase):
         B{proxy-connection} should be removed if present, and B{connection}
         should be added.
         """
-        client = ProxyClient(b'GET', b'/foo', b'HTTP/1.0',
-            {b"accept": b"text/html", b"proxy-connection": b"foo"}, b'', None)
-        self.assertEqual(client.headers,
-            {b"accept": b"text/html", b"connection": b"close"})
-
+        client = ProxyClient(
+            b"GET",
+            b"/foo",
+            b"HTTP/1.0",
+            {b"accept": b"text/html", b"proxy-connection": b"foo"},
+            b"",
+            None,
+        )
+        self.assertEqual(
+            client.headers, {b"accept": b"text/html", b"connection": b"close"}
+        )
 
     def test_keepaliveNotForwarded(self):
         """
@@ -368,16 +358,14 @@ class ProxyClientTests(TestCase):
         """
         headers = {
             b"accept": b"text/html",
-            b'keep-alive': b'300',
-            b'connection': b'keep-alive',
-            }
+            b"keep-alive": b"300",
+            b"connection": b"keep-alive",
+        }
         expectedHeaders = headers.copy()
-        expectedHeaders[b'connection'] = b'close'
-        del expectedHeaders[b'keep-alive']
-        client = ProxyClient(b'GET', b'/foo', b'HTTP/1.0', headers, b'', None)
-        self.assertForwardsHeaders(
-            client, b'GET /foo HTTP/1.0', expectedHeaders)
-
+        expectedHeaders[b"connection"] = b"close"
+        del expectedHeaders[b"keep-alive"]
+        client = ProxyClient(b"GET", b"/foo", b"HTTP/1.0", headers, b"", None)
+        self.assertForwardsHeaders(client, b"GET /foo HTTP/1.0", expectedHeaders)
 
     def test_defaultHeadersOverridden(self):
         """
@@ -385,22 +373,19 @@ class ProxyClientTests(TestCase):
         default. When we get these headers back from the remote server, the
         defaults are overridden rather than simply appended.
         """
-        request = self.makeRequest(b'foo')
-        request.responseHeaders.setRawHeaders(b'server', [b'old-bar'])
-        request.responseHeaders.setRawHeaders(b'date', [b'old-baz'])
-        request.responseHeaders.setRawHeaders(b'content-type', [b"old/qux"])
-        client = self.makeProxyClient(request, headers={b'accept': b'text/html'})
+        request = self.makeRequest(b"foo")
+        request.responseHeaders.setRawHeaders(b"server", [b"old-bar"])
+        request.responseHeaders.setRawHeaders(b"date", [b"old-baz"])
+        request.responseHeaders.setRawHeaders(b"content-type", [b"old/qux"])
+        client = self.makeProxyClient(request, headers={b"accept": b"text/html"})
         self.connectProxy(client)
         headers = {
-            b'Server': [b'bar'],
-            b'Date': [b'2010-01-01'],
-            b'Content-Type': [b'application/x-baz'],
-            }
-        client.dataReceived(
-            self.makeResponseBytes(200, b"OK", headers.items(), b''))
-        self.assertForwardsResponse(
-            request, 200, b'OK', list(headers.items()), b'')
-
+            b"Server": [b"bar"],
+            b"Date": [b"2010-01-01"],
+            b"Content-Type": [b"application/x-baz"],
+        }
+        client.dataReceived(self.makeResponseBytes(200, b"OK", headers.items(), b""))
+        self.assertForwardsResponse(request, 200, b"OK", list(headers.items()), b"")
 
 
 class ProxyClientFactoryTests(TestCase):
@@ -413,38 +398,37 @@ class ProxyClientFactoryTests(TestCase):
         Check that L{ProxyClientFactory.clientConnectionFailed} produces
         a B{501} response to the parent request.
         """
-        request = DummyRequest([b'foo'])
-        factory = ProxyClientFactory(b'GET', b'/foo', b'HTTP/1.0',
-                                     {b"accept": b"text/html"}, '', request)
+        request = DummyRequest([b"foo"])
+        factory = ProxyClientFactory(
+            b"GET", b"/foo", b"HTTP/1.0", {b"accept": b"text/html"}, "", request
+        )
 
         factory.clientConnectionFailed(None, None)
         self.assertEqual(request.responseCode, 501)
         self.assertEqual(request.responseMessage, b"Gateway error")
         self.assertEqual(
             list(request.responseHeaders.getAllRawHeaders()),
-            [(b"Content-Type", [b"text/html"])])
-        self.assertEqual(
-            b''.join(request.written),
-            b"<H1>Could not connect</H1>")
+            [(b"Content-Type", [b"text/html"])],
+        )
+        self.assertEqual(b"".join(request.written), b"<H1>Could not connect</H1>")
         self.assertEqual(request.finished, 1)
 
-
     def test_buildProtocol(self):
         """
         L{ProxyClientFactory.buildProtocol} should produce a L{ProxyClient}
         with the same values of attributes (with updates on the headers).
         """
-        factory = ProxyClientFactory(b'GET', b'/foo', b'HTTP/1.0',
-                                     {b"accept": b"text/html"}, b'Some data',
-                                     None)
+        factory = ProxyClientFactory(
+            b"GET", b"/foo", b"HTTP/1.0", {b"accept": b"text/html"}, b"Some data", None
+        )
         proto = factory.buildProtocol(None)
         self.assertIsInstance(proto, ProxyClient)
-        self.assertEqual(proto.command, b'GET')
-        self.assertEqual(proto.rest, b'/foo')
-        self.assertEqual(proto.data, b'Some data')
-        self.assertEqual(proto.headers,
-                          {b"accept": b"text/html", b"connection": b"close"})
-
+        self.assertEqual(proto.command, b"GET")
+        self.assertEqual(proto.rest, b"/foo")
+        self.assertEqual(proto.data, b"Some data")
+        self.assertEqual(
+            proto.headers, {b"accept": b"text/html", b"connection": b"close"}
+        )
 
 
 class ProxyRequestTests(TestCase):
@@ -463,23 +447,21 @@ class ProxyRequestTests(TestCase):
         request = ProxyRequest(channel, False, reactor)
         request.gotLength(len(data))
         request.handleContentChunk(data)
-        request.requestReceived(method, b'http://example.com' + uri,
-                                b'HTTP/1.0')
+        request.requestReceived(method, b"http://example.com" + uri, b"HTTP/1.0")
 
         self.assertEqual(len(reactor.tcpClients), 1)
-        self.assertEqual(reactor.tcpClients[0][0], u"example.com")
+        self.assertEqual(reactor.tcpClients[0][0], "example.com")
         self.assertEqual(reactor.tcpClients[0][1], 80)
 
         factory = reactor.tcpClients[0][2]
         self.assertIsInstance(factory, ProxyClientFactory)
         self.assertEqual(factory.command, method)
-        self.assertEqual(factory.version, b'HTTP/1.0')
-        self.assertEqual(factory.headers, {b'host': b'example.com'})
+        self.assertEqual(factory.version, b"HTTP/1.0")
+        self.assertEqual(factory.headers, {b"host": b"example.com"})
         self.assertEqual(factory.data, data)
         self.assertEqual(factory.rest, expectedURI)
         self.assertEqual(factory.father, request)
 
-
     def test_process(self):
         """
         L{ProxyRequest.process} should create a connection to the given server,
@@ -492,7 +474,6 @@ class ProxyRequestTests(TestCase):
         """
         return self._testProcess(b"/foo/bar", b"/foo/bar")
 
-
     def test_processWithoutTrailingSlash(self):
         """
         If the incoming request doesn't contain a slash,
@@ -501,15 +482,12 @@ class ProxyRequestTests(TestCase):
         """
         return self._testProcess(b"", b"/")
 
-
     def test_processWithData(self):
         """
         L{ProxyRequest.process} should be able to retrieve request body and
         to forward it.
         """
-        return self._testProcess(
-            b"/foo/bar", b"/foo/bar", b"POST", b"Some content")
-
+        return self._testProcess(b"/foo/bar", b"/foo/bar", b"POST", b"Some content")
 
     def test_processWithPort(self):
         """
@@ -521,17 +499,15 @@ class ProxyRequestTests(TestCase):
         reactor = MemoryReactor()
         request = ProxyRequest(channel, False, reactor)
         request.gotLength(0)
-        request.requestReceived(b'GET', b'http://example.com:1234/foo/bar',
-                                b'HTTP/1.0')
+        request.requestReceived(b"GET", b"http://example.com:1234/foo/bar", b"HTTP/1.0")
 
         # That should create one connection, with the port parsed from the URL
         self.assertEqual(len(reactor.tcpClients), 1)
-        self.assertEqual(reactor.tcpClients[0][0], u"example.com")
+        self.assertEqual(reactor.tcpClients[0][0], "example.com")
         self.assertEqual(reactor.tcpClients[0][1], 1234)
 
 
-
-class DummyFactory(object):
+class DummyFactory:
     """
     A simple holder for C{host} and C{port} information.
     """
@@ -541,7 +517,6 @@ class DummyFactory(object):
         self.port = port
 
 
-
 class ReverseProxyRequestTests(TestCase):
     """
     Tests for L{ReverseProxyRequest}.
@@ -558,16 +533,16 @@ class ReverseProxyRequestTests(TestCase):
         channel = DummyChannel(transport)
         reactor = MemoryReactor()
         request = ReverseProxyRequest(channel, False, reactor)
-        request.factory = DummyFactory(u"example.com", 1234)
+        request.factory = DummyFactory("example.com", 1234)
         request.gotLength(0)
-        request.requestReceived(b'GET', b'/foo/bar', b'HTTP/1.0')
+        request.requestReceived(b"GET", b"/foo/bar", b"HTTP/1.0")
 
         # Check that one connection has been created, to the good host/port
         self.assertEqual(len(reactor.tcpClients), 1)
-        self.assertEqual(reactor.tcpClients[0][0], u"example.com")
+        self.assertEqual(reactor.tcpClients[0][0], "example.com")
         self.assertEqual(reactor.tcpClients[0][1], 1234)
 
         # Check the factory passed to the connect, and its headers
         factory = reactor.tcpClients[0][2]
         self.assertIsInstance(factory, ProxyClientFactory)
-        self.assertEqual(factory.headers, {b'host': b'example.com'})
+        self.assertEqual(factory.headers, {b"host": b"example.com"})
